﻿Shader "NPR/ShadowColorizationShader"
{
	Properties
	{
		_MainTex ("Texture", 2D) = "white" {}
	}
	SubShader
	{
		// No culling or depth
		Cull Off ZWrite Off ZTest Always

		Pass
		{
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag
			
			#include "UnityCG.cginc"
			#include "colorspaces.cginc" 

			struct appdata
			{
				float4 vertex : POSITION;
				float2 uv : TEXCOORD0;
			};

			struct v2f
			{
				float2 uv : TEXCOORD0;
				float4 vertex : SV_POSITION;
			};

			v2f vert (appdata v)
			{
				v2f o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.uv = v.uv;
				return o;
			}
			
			sampler2D _MainTex;
			float luminanceThreshold;

			float luminance(float3 rgb) {
				return 0.2126*rgb.r + 0.7152*rgb.g + 0.0722*rgb.b;
			}

			float3 shadowColor(float3 rgb)
			{
				//return float3(0, 0, 0); // black shadows
				//return float3(0, 0, 0.2); // dark blue shadows
				//return rgb * 0.5; // darkened shadows

				float3 hsv = rgb_to_hsv(rgb.xyz);
				//hsv.x = 0.01; // shadows with red hue
				hsv.x = frac(hsv.x + 0.5); // complementary shadows
				//hsv.y *= 0.1; // desaturated shadows
				return hsv_to_rgb(hsv);
			}

			float4 frag (v2f i) : SV_Target
			{
				float4 col = tex2D(_MainTex, i.uv);
				if (luminance(col) <= luminanceThreshold) return float4(shadowColor(col.rgb),1);
				else return col;
			}
			ENDCG
		}
	}
}
